"""
             Yasuharu Okamoto, 2019.02
"""
import matplotlib.pyplot as plt
import numpy as np
import pymc3 as pm
import arviz as az

X = ([[0]*5 + [0]*5] * 7) + ([[1]*5 + [0]*5] * 7) +\
		([[1]*5 + [1]*5] * 7)
		
print('X =')
for v in X:
	print(v)

N = len(X)
print('N = ', N)
M = len(X[0])
print('M = ', M)
NStg = 3				#	Number of stages
print('NStg = ', NStg)

a_Diri = np.ones((M, NStg + 1))
print('a_Diri =\n', a_Diri)

a_coeff = np.zeros((NStg, NStg + 1))
for i in range(NStg):
	for j in range(i + 1):
		a_coeff[i][j] = 1
print('a_coeff =\n', a_coeff)

a_p = np.ones(NStg)
print('a_p =\n', a_p)

with pm.Model() as stage_model:
	prob_steps = pm.Dirichlet('prob_steps', a = a_Diri, shape = (M, NStg + 1))
	coeff = pm.math.constant(a_coeff, 'coeff')
	prob_response_ = pm.math.matrix_dot(coeff, prob_steps.T)
	prob_response = pm.Deterministic('prob_response', prob_response_) 
	p_stages = pm.Categorical('p_stages', p = a_p, shape = N)
	p_at_stage = pm.Deterministic('p_at_stage', prob_response[p_stages])
	y = pm.Bernoulli('y', p = prob_response[p_stages],  observed = X)
	
	trace = pm.sample()	

summary = pm.summary(trace)
print('summary...\n', summary)
ary_prob_response = trace['prob_response']
print('\nP(correct|item,stage)\n')
print('{0: >10s}'.format(' '), end = '')
for i in range(NStg):
	print("{0:>10s}".format('Stage-{}'.format(i)), end = '')
print(' ')  
for j in range(M):
	print('Item-{0: <5d}'.format(j), end = '')
	for s in range(NStg):
		print('{0: >10.5f}'.format(ary_prob_response[:,s,j].mean()), end = '')
	print(' ')

ary_p_stages = trace['p_stages']
ary_p_stages = ary_p_stages.T
print("\nMean stages and their SD's\n")
print('{0: >11s}{1:>10s}{2:>10s}'.format(' ', 'Mean', 'SD'))
for i in range(len(ary_p_stages)):
	print('person-{0: <4d}{1:>10.3f}{2:>10.3f}'.format(i, ary_p_stages[i].mean(),
													ary_p_stages[i].std()))
													
v_waic = az.waic(trace, scale = 'deviance')
print(v_waic)
print('\nWAIC = {0:.3f}  for the number of stages = {1}'.format(v_waic['waic'], NStg))
